@@ -2,7 +2,8 @@ from utils.Config import Config
 
 _cfg = Config("youtube", [
               ("cache-folder", Config.STRING, "/media/mmc1"),
-              ("hi-quality", Config.BOOL, False),
+              #("hi-quality", Config.BOOL, False),
+              ("quality-type", Config.INTEGER, 0)
               ])
 
 
@@ -25,3 +26,13 @@ def get_hi_quality():
 
     return _cfg["hi-quality"]
 
+
+def set_quality_type(t):
+
+    _cfg["quality-type"] = t
+    
+    
+def get_quality_type():
+
+    return _cfg["quality-type"]
+
